/**
 * Gulp tasks for building JavaScript.
 */
const display = require("./util/display");
const uc = require("./util/unite-config");
const moduleConfig = require("./util/module-config");
const themeUtils = require("./util/theme-utils");
const asyncUtil = require("./util/async-util");
const gulp = require("gulp");
const path = require("path");
const fs = require("fs");
const util = require("util");
const del = require("del");
const runSequence = require("run-sequence");
const packageJson = require("../../package.json");
const htmlMin = require("gulp-htmlmin");
const deleteEmpty = require("delete-empty");
require("./build-transpile");
require("./build-bundle-app");
require("./build-bundle-vendor");
require("./build-lint");
require("./build-css-app");
require("./build-css-components");
require("./build-css-post-app");
require("./build-css-post-components");

gulp.task("build-clean", async () => {
    const uniteConfig = await uc.getUniteConfig();
    const toClean = [
        path.join(uniteConfig.directories.dist, "**/*"),
        path.join(uniteConfig.directories.cssDist, "**/*"),
        "./index.html"
    ];
    display.info("Cleaning", toClean);
    return del(toClean);
});

gulp.task("build-copy-index", async () => {
    display.info("Building Index Page");

    const uniteConfig = await uc.getUniteConfig();
    const uniteThemeConfig = await uc.getUniteThemeConfig();
    const buildConfiguration = uc.getBuildConfiguration(uniteConfig);

    return themeUtils.buildIndex(uniteConfig, uniteThemeConfig, buildConfiguration, packageJson);
});

gulp.task("build-post-clean", async () => {
    const uniteConfig = await uc.getUniteConfig();
    const buildConfiguration = uc.getBuildConfiguration(uniteConfig);

    const toClean = [];

    if (uniteConfig.cssPost === "None") {
        toClean.push(path.join(uniteConfig.directories.cssDist, "**/main.css"));
    } else {
        toClean.push(path.join(uniteConfig.directories.cssDist, "**/!(style).css"));
    }
    if (buildConfiguration.bundle) {
        toClean.push(path.join(uniteConfig.directories.dist, "**/!(app-bundle|vendor-bundle).*"));
    }
    display.info("Cleaning", toClean);
    try {
        await del(toClean);
        await util.promisify(deleteEmpty)(uniteConfig.directories.dist, {"verbose": false});
    } catch (err) {
        display.error(err);
        process.exit(1);
    }
});

gulp.task("build-index-min", async () => {
    const uniteConfig = await uc.getUniteConfig();
    const buildConfiguration = uc.getBuildConfiguration(uniteConfig);

    if (buildConfiguration.minify) {
        return asyncUtil.stream(gulp.src("./index.html")
            .pipe(htmlMin({"collapseWhitespace": true, "removeComments": true}))
            .pipe(gulp.dest("./")));
    }
});

gulp.task("build-html-min", async () => {
    const uniteConfig = await uc.getUniteConfig();
    const buildConfiguration = uc.getBuildConfiguration(uniteConfig);

    if (buildConfiguration.minify) {
        return asyncUtil.stream(gulp.src(path.join(uniteConfig.directories.dist, "**/*.html"))
            .pipe(htmlMin({"collapseWhitespace": true, "removeComments": true}))
            .pipe(gulp.dest(uniteConfig.directories.dist)));
    }
});

gulp.task("build-copy-components", async () => {
    const uniteConfig = await uc.getUniteConfig();

    return asyncUtil.stream(gulp.src([path.join(uniteConfig.directories.src, "**/*.html")])
        .pipe(gulp.dest(uniteConfig.directories.dist)));
});

gulp.task("build-module-config", async () => {
    const uniteConfig = await uc.getUniteConfig();
    const buildConfiguration = uc.getBuildConfiguration(uniteConfig);
    const config = moduleConfig.create(uniteConfig, ["app", "both"], buildConfiguration.bundle);

    try {
        await util.promisify(fs.writeFile)(path.join(uniteConfig.directories.dist, "app-module-config.js"), config);
    } catch (err) {
        display.error("Writing app-module-config.js", err);
        process.exit(1);
    }
});

gulp.task("build", async () => {
    try {
        await util.promisify(runSequence)("build-clean",
            "build-transpile",
            "build-lint",
            "build-css-app",
            "build-css-post-app",
            "build-css-components",
            "build-css-post-components",
            "build-copy-components",
            "build-module-config",
            "build-html-min",
            "build-bundle-vendor",
            "build-bundle-app",
            "build-copy-index",
            "build-index-min",
            "build-post-clean");
    } catch (err) {
        display.error("Unhandled error during task", err);
        process.exit(1);
    }
});

/* Generated by UniteJS */
